@@ -1,6 +1,6 @@
 /* gpg-agent.c  -  The GnuPG Agent
  * Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005,
- *               2006, 2007 Free Software Foundation, Inc.
+ *               2006, 2007, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -141,6 +141,9 @@ static ARGPARSE_OPTS opts[] = {
   { oDisableScdaemon, "disable-scdaemon", 0, N_("do not use the SCdaemon") },
   { oFakedSystemTime, "faked-system-time", 2, "@" }, /* (epoch time) */
 
+  { oBatch,      "batch",       0, "@" },
+  { oHomedir,    "homedir",     2, "@"},   
+
   { oDisplay,    "display",     2, "@" },
   { oTTYname,    "ttyname",     2, "@" },
   { oTTYtype,    "ttytype",     2, "@" },
@@ -2019,7 +2022,7 @@ check_own_socket_thread (void *arg)
       log_error ("socket is now serviced by another server\n");
       rc = 1;
     }
-  else if (opt.verbose)
+  else if (opt.verbose > 1)
     log_error ("socket is still served by this server\n");
     
   xfree (buffer);
@@ -2065,7 +2068,7 @@ check_own_socket (void)
   tattr = pth_attr_new();
   pth_attr_set (tattr, PTH_ATTR_JOINABLE, 0);
   pth_attr_set (tattr, PTH_ATTR_STACK_SIZE, 256*1024);
-  pth_attr_set (tattr, PTH_ATTR_NAME, "check-owb-socket");
+  pth_attr_set (tattr, PTH_ATTR_NAME, "check-own-socket");
 
   if (!pth_spawn (tattr, check_own_socket_thread, sockname))
       log_error ("error spawning check_own_socket_thread: %s\n",
