@@ -208,6 +208,17 @@ store_serialno (const char *line)
 
 
 
+/* This is a dummy data line callback.  */
+static int
+dummy_data_cb (void *opaque, const void *buffer, size_t length)
+{
+  (void)opaque;
+  (void)buffer;
+  (void)length;
+  return 0;
+}
+
+
 /* This is the default inquiry callback.  It mainly handles the
    Pinentry notifications.  */
 static int
@@ -239,6 +250,7 @@ agent_release_card_info (struct agent_card_info_s *info)
     return;
 
   xfree (info->serialno); info->serialno = NULL;
+  xfree (info->apptype); info->apptype = NULL;
   xfree (info->disp_name); info->disp_name = NULL;
   xfree (info->disp_lang); info->disp_lang = NULL;
   xfree (info->pubkey_url); info->pubkey_url = NULL;
@@ -267,6 +279,11 @@ learn_status_cb (void *opaque, const char *line)
       parm->is_v2 = (strlen (parm->serialno) >= 16 
                      && xtoi_2 (parm->serialno+12) >= 2 );
     }
+  else if (keywordlen == 7 && !memcmp (keyword, "APPTYPE", keywordlen))
+    {
+      xfree (parm->apptype);
+      parm->apptype = unescape_status_string (line);
+    }
   else if (keywordlen == 9 && !memcmp (keyword, "DISP-NAME", keywordlen))
     {
       xfree (parm->disp_name);
@@ -372,7 +389,7 @@ agent_learn (struct agent_card_info_s *info)
 
   memset (info, 0, sizeof *info);
   rc = assuan_transact (agent_ctx, "LEARN --send",
-                        NULL, NULL, default_inq_cb, NULL,
+                        dummy_data_cb, NULL, default_inq_cb, NULL,
                         learn_status_cb, info);
   
   return rc;
@@ -681,6 +698,15 @@ agent_scd_pksign (const char *serialno, int hashalgo,
   if (indatalen*2 + 50 > DIM(line))
     return gpg_error (GPG_ERR_GENERAL);
 
+  /* Send the serialno command to initialize the connection. We don't
+     care about the data returned.  If the card has already been
+     initialized, this is a very fast command.  We request the openpgp
+     card because that is waht we expect. */
+  rc = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
+                        NULL, NULL, NULL, NULL, NULL, NULL);
+  if (rc)
+    return rc;
+
   sprintf (line, "SCD SETDATA ");
   p = line + strlen (line);
   for (i=0; i < indatalen ; i++, p += 2 )
@@ -737,6 +763,15 @@ agent_scd_pkdecrypt (const char *serialno,
   if (indatalen*2 + 50 > DIM(line))
     return gpg_error (GPG_ERR_GENERAL);
 
+  /* Send the serialno command to initialize the connection. We don't
+     care about the data returned.  If the card has already been
+     initialized, this is a very fast command.  We request the openpgp
+     card because that is waht we expect. */
+  rc = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
+                        NULL, NULL, NULL, NULL, NULL, NULL);
+  if (rc)
+    return rc;
+
   sprintf (line, "SCD SETDATA ");
   p = line + strlen (line);
   for (i=0; i < indatalen ; i++, p += 2 )
