@@ -48,6 +48,8 @@ progress_cb (void *ctx, const char *what, int printchar,
 {
   char buf[50];
 
+  (void)ctx;
+
   if ( printchar == '\n' && !strcmp (what, "primegen") )
     snprintf (buf, sizeof buf -1, "%.20s X 100 100", what );
   else
@@ -155,6 +157,19 @@ write_status_text ( int no, const char *text)
 }
 
 
+void
+write_status_error (const char *where, int errcode)
+{
+  if (!statusfp || !status_currently_allowed (STATUS_ERROR))
+    return;  /* Not enabled or allowed. */
+
+  fprintf (statusfp, "[GNUPG:] %s %s %u\n", 
+           get_status_string (STATUS_ERROR), where, gpg_err_code (errcode));
+  if (fflush (statusfp) && opt.exit_on_status_write_error)
+    g10_exit (0);
+}
+
+
 /*
  * Write a status line with a buffer using %XX escapes.  If WRAP is >
  * 0 wrap the line after this length.  If STRING is not NULL it will
@@ -227,7 +242,7 @@ write_status_buffer ( int no, const char *buffer, size_t len, int wrap )
 
 
 /* Print the BEGIN_SIGNING status message.  If MD is not NULL it is
-   used retrieve the hash algorithms used for the message. */
+   used to retrieve the hash algorithms used for the message. */
 void
 write_status_begin_signing (gcry_md_hd_t md)
 {
@@ -342,10 +357,6 @@ cpr_enabled()
 {
     if( opt.command_fd != -1 )
 	return 1;
-#ifdef USE_SHM_COPROCESSING
-    if( opt.shm_coprocess )
-	return 1;
-#endif
     return 0;
 }
 
@@ -356,10 +367,6 @@ cpr_get_no_help( const char *keyword, const char *prompt )
 
     if( opt.command_fd != -1 )
 	return do_get_from_fd ( keyword, 0, 0 );
-#ifdef USE_SHM_COPROCESSING
-    if( opt.shm_coprocess )
-	return do_shm_get( keyword, 0, 0 );
-#endif
     for(;;) {
 	p = tty_get( prompt );
         return p;
@@ -373,10 +380,6 @@ cpr_get( const char *keyword, const char *prompt )
 
     if( opt.command_fd != -1 )
 	return do_get_from_fd ( keyword, 0, 0 );
-#ifdef USE_SHM_COPROCESSING
-    if( opt.shm_coprocess )
-	return do_shm_get( keyword, 0, 0 );
-#endif
     for(;;) {
 	p = tty_get( prompt );
 	if( *p=='?' && !p[1] && !(keyword && !*keyword)) {
@@ -409,10 +412,6 @@ cpr_get_hidden( const char *keyword, const char *prompt )
 
     if( opt.command_fd != -1 )
 	return do_get_from_fd ( keyword, 1, 0 );
-#ifdef USE_SHM_COPROCESSING
-    if( opt.shm_coprocess )
-	return do_shm_get( keyword, 1, 0 );
-#endif
     for(;;) {
 	p = tty_get_hidden( prompt );
 	if( *p == '?' && !p[1] ) {
@@ -429,10 +428,6 @@ cpr_kill_prompt(void)
 {
     if( opt.command_fd != -1 )
 	return;
-#ifdef USE_SHM_COPROCESSING
-    if( opt.shm_coprocess )
-	return;
-#endif
     tty_kill_prompt();
     return;
 }
@@ -445,10 +440,6 @@ cpr_get_answer_is_yes( const char *keyword, const char *prompt )
 
     if( opt.command_fd != -1 )
 	return !!do_get_from_fd ( keyword, 0, 1 );
-#ifdef USE_SHM_COPROCESSING
-    if( opt.shm_coprocess )
-	return !!do_shm_get( keyword, 0, 1 );
-#endif
     for(;;) {
 	p = tty_get( prompt );
 	trim_spaces(p); /* it is okay to do this here */
@@ -473,10 +464,6 @@ cpr_get_answer_yes_no_quit( const char *keyword, const char *prompt )
 
     if( opt.command_fd != -1 )
 	return !!do_get_from_fd ( keyword, 0, 1 );
-#ifdef USE_SHM_COPROCESSING
-    if( opt.shm_coprocess )
-	return !!do_shm_get( keyword, 0, 1 );
-#endif
     for(;;) {
 	p = tty_get( prompt );
 	trim_spaces(p); /* it is okay to do this here */
@@ -505,10 +492,6 @@ cpr_get_answer_okay_cancel (const char *keyword,
 
   if( opt.command_fd != -1 )
     answer = do_get_from_fd ( keyword, 0, 0 );
-#ifdef USE_SHM_COPROCESSING
-  else if( opt.shm_coprocess )
-    answer = do_shm_get( keyword, 0, 0 );
-#endif
 
   if (answer)
     {
