@@ -1,6 +1,6 @@
 /* keyserver.c - generic keyserver code
  * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006,
- *               2007 Free Software Foundation, Inc.
+ *               2007, 2008 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -330,7 +330,15 @@ parse_keyserver_uri(const char *string,int require_scheme,
       keyserver->scheme=xstrdup("hkp");
     }
 
-  if(assume_hkp || (uri[0]=='/' && uri[1]=='/'))
+  if (uri[0]=='/' && uri[1]=='/' && uri[2] == '/')
+    {
+      /* Three slashes means network path with a default host name.
+         This is a hack because it does not crok all possible
+         combiantions.  We should better repalce all code bythe parser
+         from http.c.  */
+      keyserver->path = xstrdup (uri+2);
+    }
+  else if(assume_hkp || (uri[0]=='/' && uri[1]=='/'))
     {
       /* Two slashes means network path. */
 
@@ -683,9 +691,9 @@ parse_keyrec(char *keystring)
       while(*tok)
 	if(tok[0]=='%' && tok[1] && tok[2])
 	  {
-	    if((userid[i]=hextobyte(&tok[1]))==-1)
-	      userid[i]='?';
+            int c;
 
+	    userid[i] = (c=hextobyte(&tok[1])) == -1 ? '?' : c;
 	    i++;
 	    tok+=3;
 	  }
@@ -723,6 +731,8 @@ show_prompt(KEYDB_SEARCH_DESC *desc,int numdesc,int count,const char *search)
 {
   char *answer;
 
+  fflush (stdout);
+
   if(count && opt.command_fd==-1)
     {
       static int from=1;
@@ -2000,7 +2010,14 @@ keyserver_import_cert(const char *name,unsigned char **fpr,size_t *fpr_len)
     *domain='.';
 
   type=get_dns_cert(look,max_cert_size,&key,fpr,fpr_len,&url);
-  if(type==1)
+  if (!type || type == -1)
+    {
+      /* There might be an error in res_query which leads to an error
+         return (-1) in the case that nothing was found.  Thus we take
+         all errors as key not found.  */
+      rc = G10ERR_NO_PUBKEY;
+    }
+  else if (type==1)
     {
       int armor_status=opt.no_armor;
 
@@ -2026,13 +2043,7 @@ keyserver_import_cert(const char *name,unsigned char **fpr,size_t *fpr_len)
 	  spec=parse_keyserver_uri(url,1,NULL,0);
 	  if(spec)
 	    {
-	      strlist_t list=NULL;
-
-	      add_to_strlist(&list,url);
-
-	      rc=keyserver_fetch(list);
-
-	      free_strlist(list);
+	      rc=keyserver_import_fprint(*fpr,*fpr_len,spec);
 	      free_keyserver_spec(spec);
 	    }
 	}
@@ -2064,26 +2075,30 @@ int
 keyserver_import_pka(const char *name,unsigned char **fpr,size_t *fpr_len)
 {
   char *uri;
-  int rc=-1;
+  int rc = G10ERR_NO_PUBKEY;
 
-  *fpr=xmalloc(20);
-  *fpr_len=20;
+  *fpr = xmalloc (20);
+  *fpr_len = 20;
 
   uri = get_pka_info (name, *fpr);
-  if (uri)
+  if (uri && *uri)
     {
+      /* An URI is available.  Lookup the key. */
       struct keyserver_spec *spec;
       spec = parse_keyserver_uri (uri, 1, NULL, 0);
       if (spec)
 	{
-	  rc=keyserver_import_fprint (*fpr, 20, spec);
+	  rc = keyserver_import_fprint (*fpr, 20, spec);
 	  free_keyserver_spec (spec);
 	}
       xfree (uri);
     }
 
-  if(rc!=0)
-    xfree(*fpr);
+  if (rc)
+    {
+      xfree(*fpr);
+      *fpr = NULL;
+    }
 
   return rc;
 }
