@@ -377,6 +377,8 @@ passphrase_get ( u32 *keyid, int mode, const char *cacheid,
          errors from the agent.  */ 
       if (canceled)
         *canceled = 1;
+
+      write_status_error ("get_passphrase", rc);
     }
 
   if (pk)
@@ -398,6 +400,8 @@ void
 passphrase_clear_cache ( u32 *keyid, const char *cacheid, int algo )
 {
   int rc;
+
+  (void)algo;
     
   if (!cacheid)
     {
@@ -441,6 +445,8 @@ ask_passphrase (const char *description,
                 const char *cacheid, int *canceled)
 {
   char *pw = NULL;
+
+  (void)promptid;
   
   if (canceled)
     *canceled = 0;
