@@ -114,6 +114,11 @@ struct server_local_s
 
   /* A disconnect command has been sent.  */
   int disconnect_allowed;
+
+  /* If set to true we will be terminate ourself at the end of the
+     this session.  */
+  int stopme;  
+
 };
 
 
@@ -469,7 +474,7 @@ percent_plus_unescape (unsigned char *string)
 /* SERIALNO [APPTYPE] 
 
    Return the serial number of the card using a status reponse.  This
-   functon should be used to check for the presence of a card.
+   function should be used to check for the presence of a card.
 
    If APPTYPE is given, an application of that type is selected and an
    error is returned if the application is not supported or available.
@@ -543,7 +548,8 @@ cmd_serialno (assuan_context_t ctx, char *line)
        P15     = PKCS-15 structure used
        DINSIG  = DIN SIG
        OPENPGP = OpenPGP card
- 
+       NKS     = NetKey card
+
    are implemented.  These strings are aliases for the AID
 
      S KEYPAIRINFO <hexstring_with_keygrip> <hexstring_with_id>
@@ -1556,6 +1562,13 @@ cmd_unlock (assuan_context_t ctx, char *line)
 
    reader_list - Return a list of detected card readers.  Does
                  currently only work with the internal CCID driver.
+
+   deny_admin  - Returns OK if admin commands are not allowed or
+                 GPG_ERR_GENERAL if admin commands are allowed.
+
+   app_list    - Return a list of supported applciations.  One
+                 application per line, fields delimited by colons,
+                 first field is the name.
 */
 
 static int
@@ -1621,6 +1634,17 @@ cmd_getinfo (assuan_context_t ctx, char *line)
         rc = gpg_error (GPG_ERR_NO_DATA);
       xfree (s);
     }
+  else if (!strcmp (line, "deny_admin"))
+    rc = opt.allow_admin? gpg_error (GPG_ERR_GENERAL) : 0;
+  else if (!strcmp (line, "app_list"))
+    {
+      char *s = get_supported_applications ();
+      if (s)
+        rc = assuan_send_data (ctx, s, strlen (s));
+      else
+        rc = 0;
+      xfree (s);
+    }
   else
     rc = set_error (GPG_ERR_ASS_PARAMETER, "unknown value for WHAT");
   return rc;
@@ -1760,6 +1784,17 @@ cmd_apdu (assuan_context_t ctx, char *line)
 }
 
 
+/* KILLSCD - Commit suicide. */
+static int
+cmd_killscd (assuan_context_t ctx, char *line)
+{
+  ctrl_t ctrl = assuan_get_pointer (ctx);
+
+  (void)line;
+
+  ctrl->server_local->stopme = 1;
+  return gpg_error (GPG_ERR_EOF);
+}
 
 
 
@@ -1795,6 +1830,7 @@ register_commands (assuan_context_t ctx)
     { "RESTART",      cmd_restart },
     { "DISCONNECT",   cmd_disconnect },
     { "APDU",         cmd_apdu },
+    { "KILLSCD",      cmd_killscd },
     { NULL }
   };
   int i, rc;
@@ -1821,6 +1857,7 @@ scd_command_handler (ctrl_t ctrl, int fd)
 {
   int rc;
   assuan_context_t ctx;
+  int stopme;
   
   if (fd == -1)
     {
@@ -1906,12 +1943,16 @@ scd_command_handler (ctrl_t ctrl, int fd)
           BUG ();
       sl->next_session = ctrl->server_local->next_session;
     }
+  stopme = ctrl->server_local->stopme;
   xfree (ctrl->server_local);
   ctrl->server_local = NULL;
 
   /* Release the Assuan context.  */
   assuan_deinit_server (ctx);
 
+  if (stopme)
+    scd_exit (0);
+
   /* If there are no more sessions return true.  */
   return !session_list;
 }
