@@ -1,6 +1,6 @@
 /* scdaemon.c  -  The GnuPG Smartcard Daemon
  * Copyright (C) 2001, 2002, 2004, 2005, 
- *               2007, 2008 Free Software Foundation, Inc.
+ *               2007, 2008, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -69,6 +69,7 @@ enum cmd_and_opt_values
   oDebugWait,
   oDebugAllowCoreDump,
   oDebugCCIDDriver,
+  oDebugLogTid,
   oNoGreeting,
   oNoOptions,
   oHomedir,
@@ -117,6 +118,7 @@ static ARGPARSE_OPTS opts[] = {
   ARGPARSE_s_n (oDebugAllowCoreDump, "debug-allow-core-dump", "@"),
   ARGPARSE_s_n (oDebugCCIDDriver, "debug-ccid-driver", "@"),
   ARGPARSE_s_n (oDebugDisableTicker, "debug-disable-ticker", "@"),
+  ARGPARSE_s_n (oDebugLogTid, "debug-log-tid", "@"),
   ARGPARSE_s_n (oNoDetach, "no-detach", N_("do not detach from the console")),
   ARGPARSE_s_s (oLogFile,  "log-file", N_("|FILE|write a log to FILE")),
   ARGPARSE_s_s (oReaderPort, "reader-port", 
@@ -136,9 +138,9 @@ static ARGPARSE_OPTS opts[] = {
                 N_("|N|disconnect the card after N seconds of inactivity")),
   ARGPARSE_s_n (oDisableKeypad, "disable-keypad", 
                 N_("do not use a reader's keypad")),
-  ARGPARSE_s_n (oAllowAdmin, "allow-admin", 
-                N_("allow the use of admin card commands")),
-  ARGPARSE_s_n (oDenyAdmin, "deny-admin", "@"),  
+  ARGPARSE_s_n (oAllowAdmin, "allow-admin", "@"),
+  ARGPARSE_s_n (oDenyAdmin, "deny-admin", 
+                N_("deny the use of admin card commands")),
   ARGPARSE_s_s (oDisableApplication, "disable-application", "@"),
   
   ARGPARSE_end ()
@@ -263,6 +265,17 @@ my_strusage (int level)
 }
 
 
+static unsigned long
+tid_log_callback (void)
+{
+#ifdef PTH_HAVE_PTH_THREAD_ID
+  return pth_thread_id ();
+#else
+  return (unsigned long)pth_self ();
+#endif
+}
+
+
 
 
 
@@ -399,6 +412,7 @@ main (int argc, char **argv )
   may_coredump = disable_core_dumps ();
 
   /* Set default options. */
+  opt.allow_admin = 1;
   opt.pcsc_driver = DEFAULT_PCSC_DRIVER; 
 
 #ifdef HAVE_W32_SYSTEM
@@ -504,6 +518,9 @@ main (int argc, char **argv )
 #endif /*HAVE_LIBUSB*/
           break;
         case oDebugDisableTicker: ticker_disabled = 1; break;
+        case oDebugLogTid: 
+          log_set_get_tid_callback (tid_log_callback);
+          break;
 
         case oOptions:
           /* config files may not be nested (silently ignore them) */
@@ -534,7 +551,8 @@ main (int argc, char **argv )
 
         case oDisableKeypad: opt.disable_keypad = 1; break;
 
-        case oAllowAdmin: opt.allow_admin = 1; break;
+        case oAllowAdmin: /* Dummy because allow is now the default.  */
+          break;
         case oDenyAdmin: opt.allow_admin = 0; break;
           
         case oCardTimeout: opt.card_timeout = pargs.r.ret_ulong; break;
@@ -584,14 +602,6 @@ main (int argc, char **argv )
 
   set_debug (debug_level);
 
-  if (debug_wait && pipe_server)
-    {
-      log_debug ("waiting for debugger - my pid is %u .....\n",
-                 (unsigned int)getpid());
-      gnupg_sleep (debug_wait);
-      log_debug ("... okay\n");
-    }
-  
   initialize_module_command ();
 
   if (gpgconf_list == 2)
@@ -629,20 +639,28 @@ main (int argc, char **argv )
 #ifdef HAVE_LIBUSB
       printf ("disable-ccid:%lu:\n", GC_OPT_FLAG_NONE );
 #endif
-      printf ("allow-admin:%lu:\n", GC_OPT_FLAG_NONE );
+      printf ("deny-admin:%lu:\n", GC_OPT_FLAG_NONE );
       printf ("disable-keypad:%lu:\n", GC_OPT_FLAG_NONE );
       printf ("card-timeout:%lu:%d:\n", GC_OPT_FLAG_DEFAULT, 0);
 
       scd_exit (0);
     }
 
-  /* now start with logging to a file if this is desired */
+  /* Now start with logging to a file if this is desired.  */
   if (logfile)
     {
       log_set_file (logfile);
       log_set_prefix (NULL, 1|2|4);
     }
 
+  if (debug_wait && pipe_server)
+    {
+      log_debug ("waiting for debugger - my pid is %u .....\n",
+                 (unsigned int)getpid());
+      gnupg_sleep (debug_wait);
+      log_debug ("... okay\n");
+    }
+  
   if (pipe_server)
     { 
       /* This is the simple pipe based server */
@@ -1073,7 +1091,7 @@ start_connection_thread (void *arg)
               FD2INT(ctrl->thread_startup.fd));
 
   /* If this is a pipe server, we request a shutdown if the command
-     hanlder asked for it.  With the next ticker event and given that
+     handler asked for it.  With the next ticker event and given that
      no other connections are running the shutdown will then
      happen.  */
   if (scd_command_handler (ctrl, FD2INT(ctrl->thread_startup.fd))
