@@ -1,5 +1,5 @@
 /* certreqgen.c - Generate a key and a certification request
- *	Copyright (C) 2002, 2003, 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2002, 2003, 2005, 2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -247,7 +247,7 @@ get_parameter_uint (struct para_data_s *para, enum para_name key)
 /* Read the certificate generation parameters from FP and generate
    (all) certificate requests.  */
 static int
-read_parameters (ctrl_t ctrl, FILE *fp, ksba_writer_t writer)
+read_parameters (ctrl_t ctrl, estream_t fp, ksba_writer_t writer)
 {
   static struct {
     const char *name;
@@ -275,7 +275,7 @@ read_parameters (ctrl_t ctrl, FILE *fp, ksba_writer_t writer)
 
   err = NULL;
   para = NULL;
-  while (fgets (line, DIM(line)-1, fp) )
+  while (es_fgets (line, DIM(line)-1, fp) )
     {
       char *keyword, *value;
 
@@ -391,7 +391,7 @@ read_parameters (ctrl_t ctrl, FILE *fp, ksba_writer_t writer)
       log_error ("line %d: %s\n", outctrl.lnr, err);
       rc = gpg_error (GPG_ERR_GENERAL);
     }
-  else if (ferror(fp))
+  else if (es_ferror(fp))
     {
       log_error ("line %d: read error: %s\n", outctrl.lnr, strerror(errno) );
       rc = gpg_error (GPG_ERR_GENERAL);
@@ -786,8 +786,9 @@ create_request (ctrl_t ctrl,
               goto leave;
             }
           gcry_sexp_release (s_pkey);
-          for (n=0; n < 20; n++)
-            sprintf (hexgrip+n*2, "%02X", grip[n]);
+          bin2hex (grip, 20, hexgrip);
+
+          log_info ("about to sign CSR for key: &%s\n", hexgrip); 
 
           if (carddirect)
             rc = gpgsm_scd_pksign (ctrl, carddirect, NULL,
@@ -796,11 +797,23 @@ create_request (ctrl_t ctrl,
                                      GCRY_MD_SHA1,
                                      &sigval, &siglen);
           else
-            rc = gpgsm_agent_pksign (ctrl, hexgrip, NULL,
-                                     gcry_md_read(md, GCRY_MD_SHA1), 
-                                     gcry_md_get_algo_dlen (GCRY_MD_SHA1),
-                                     GCRY_MD_SHA1,
-                                     &sigval, &siglen);
+            {
+              char *orig_codeset;
+              char *desc;
+
+              orig_codeset = i18n_switchto_utf8 ();
+              desc = percent_plus_escape 
+                (_("To complete this certificate request please enter"
+                   " the passphrase for the key you just created once"
+                   " more.\n"));
+              i18n_switchback (orig_codeset);
+              rc = gpgsm_agent_pksign (ctrl, hexgrip, desc,
+                                       gcry_md_read(md, GCRY_MD_SHA1), 
+                                       gcry_md_get_algo_dlen (GCRY_MD_SHA1),
+                                       GCRY_MD_SHA1,
+                                       &sigval, &siglen);
+              xfree (desc);
+            }
           if (rc)
             {
               log_error ("signing failed: %s\n", gpg_strerror (rc));
@@ -819,7 +832,7 @@ create_request (ctrl_t ctrl,
         }
     }
   while (stopreason != KSBA_SR_READY);   
-  
+
 
  leave:
   gcry_md_close (md);
@@ -829,27 +842,15 @@ create_request (ctrl_t ctrl,
 
 
 
-/* Create a new key by reading the parameters from in_fd or in_stream.
-   Multiple keys may be created */
+/* Create a new key by reading the parameters from IN_FP.  Multiple
+   keys may be created */
 int
-gpgsm_genkey (ctrl_t ctrl, int in_fd, FILE *in_stream, FILE *out_fp)
+gpgsm_genkey (ctrl_t ctrl, estream_t in_stream, FILE *out_fp)
 {
   int rc;
-  FILE *in_fp;
   Base64Context b64writer = NULL;
   ksba_writer_t writer;
 
-  if (in_stream)
-    in_fp = in_stream;
-  else
-    in_fp = fdopen (dup (in_fd), "rb");
-  if (!in_fp)
-    {
-      gpg_error_t tmperr = gpg_error (gpg_err_code_from_errno (errno));
-      log_error ("fdopen() failed: %s\n", strerror (errno));
-      return tmperr;
-    }
-
   ctrl->pem_name = "CERTIFICATE REQUEST";
   rc = gpgsm_create_writer (&b64writer, ctrl, out_fp, NULL, &writer);
   if (rc)
@@ -858,7 +859,7 @@ gpgsm_genkey (ctrl_t ctrl, int in_fd, FILE *in_stream, FILE *out_fp)
       goto leave;
     }
 
-  rc = read_parameters (ctrl, in_fp, writer);
+  rc = read_parameters (ctrl, in_stream, writer);
   if (rc)
     {
       log_error ("error creating certificate request: %s <%s>\n",
@@ -878,8 +879,6 @@ gpgsm_genkey (ctrl_t ctrl, int in_fd, FILE *in_stream, FILE *out_fp)
 
  leave:
   gpgsm_destroy_writer (b64writer);
-  if (!in_stream)
-    fclose (in_fp);
   return rc;
 }
 
