@@ -1,5 +1,5 @@
 /* gpgconf.c - Configuration utility for GnuPG
- *	Copyright (C) 2003, 2007 Free Software Foundation, Inc.
+ * Copyright (C) 2003, 2007, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -47,7 +47,8 @@ enum cmd_and_opt_values
     aApplyDefaults,
     aListConfig,
     aCheckConfig,
-    aListDirs
+    aListDirs,
+    aReload
 
   };
 
@@ -70,6 +71,7 @@ static ARGPARSE_OPTS opts[] =
       N_("list global configuration file") },
     { aCheckConfig,   "check-config", 256,
       N_("check global configuration file") },
+    { aReload,        "reload", 256, "@" },
 
     { 301, NULL, 0, N_("@\nOptions:\n ") },
     
@@ -176,6 +178,7 @@ main (int argc, char **argv)
         case aApplyDefaults:
         case aListConfig:
         case aCheckConfig:
+        case aReload:
 	  cmd = pargs.r_opt;
 	  break;
 
@@ -233,6 +236,31 @@ main (int argc, char **argv)
 	}
       break;
 
+    case aReload:
+      if (!fname)
+	{
+          /* Reload all.  */
+          gc_component_reload (-1);
+	}
+      else
+        {
+          /* Reload given component.  */
+          int idx;
+
+          idx = gc_component_find (fname);
+          if (idx < 0)
+            {
+              fputs (_("Component not found"), stderr);
+              putc ('\n', stderr);
+              exit (1);
+            }
+          else
+            {
+              gc_component_reload (idx);
+            }
+        }
+      break;
+
     case aListConfig:
       if (gc_process_gpgconf_conf (fname, 0, 0, get_outfp (&outfp)))
         exit (1);
@@ -258,12 +286,51 @@ main (int argc, char **argv)
       break;
       
     case aListDirs:
-      /* Show the system configuration directory for gpgconf.  */
+      /* Show the system configuration directories for gpgconf.  */
       get_outfp (&outfp);
       fprintf (outfp, "sysconfdir:%s\n",
 	       gc_percent_escape (gnupg_sysconfdir ()));
       fprintf (outfp, "bindir:%s\n",
 	       gc_percent_escape (gnupg_bindir ()));
+      fprintf (outfp, "libexecdir:%s\n",
+	       gc_percent_escape (gnupg_libexecdir ()));
+      fprintf (outfp, "libdir:%s\n",
+	       gc_percent_escape (gnupg_libdir ()));
+      fprintf (outfp, "datadir:%s\n",
+	       gc_percent_escape (gnupg_datadir ()));
+      fprintf (outfp, "localedir:%s\n",
+	       gc_percent_escape (gnupg_localedir ()));
+      fprintf (outfp, "dirmngr-socket:%s\n",
+	       gc_percent_escape (dirmngr_socket_name ()));
+      {
+        char *infostr = getenv ("GPG_AGENT_INFO");
+
+        if (!infostr || !*infostr)
+          infostr = make_filename (default_homedir (), "S.gpg-agent", NULL);
+        else
+          {
+            char *tmp;
+
+            infostr = xstrdup (infostr);
+            tmp = strchr (infostr, PATHSEP_C);
+            if (!tmp || tmp == infostr)
+              {
+                xfree (infostr);
+                infostr = NULL;
+              }
+            else
+              *tmp = 0;
+          }
+        fprintf (outfp, "agent-socket:%s\n",
+                 infostr? gc_percent_escape (infostr) : "");
+        xfree (infostr);
+      }
+      {
+        /* We need to use make_filename to expand a possible "~/".  */
+        char *tmp = make_filename (default_homedir (), NULL);
+        fprintf (outfp, "homedir:%s\n", gc_percent_escape (tmp));
+        xfree (tmp);
+      }
       break;
     }
 
